@@ -72,6 +72,7 @@ typedef struct _CloudPinyinEngine {
     const char* RequestPinyin;
     void (*ParseKey)(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue);
     char* (*ParsePinyin)(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue);
+    boolean supportSeparator;
 } CloudPinyinEngine;
 
 static void* CloudPinyinCreate(FcitxInstance* instance);
@@ -119,13 +120,15 @@ CloudPinyinEngine engine[] =
         NULL,
         "https://www.google.com/inputtools/request?ime=pinyin&text=%s",
         NULL,
-        GoogleParsePinyin
+        GoogleParsePinyin,
+        true
     },
     {
         NULL,
         "http://olime.baidu.com/py?py=%s&rn=0&pn=1&ol=1",
         NULL,
-        BaiduParsePinyin
+        BaiduParsePinyin,
+        true
     }
 };
 
@@ -870,27 +873,45 @@ char *GetCurrentString(FcitxCloudPinyin* cloudpinyin, char **ascii_part)
     size_t plength = hzlength;
     strncpy(p, string, hzlength);
     p[hzlength] = '\0';
+    // lastpos points to the start of a pinyin
+    // pinyin points to the end of current pinyin
+    // for example
+    // xi'an
+    // | |
+    // l p
+    // and we check the separator by supportSeparator for each engine.
+    // shuangpin-libpinyin returns full pinyin in preedit, so we also treat space as separator.
     do {
         endflag = (*pinyin != '\0');
         if (*pinyin == ' ' || *pinyin == '\'' || *pinyin == '\0') {
-            *pinyin = 0;
+            boolean isSeparator = false;
+
+            // skip all continous separator
+            while (*pinyin == ' ' || *pinyin == '\'') {
+                isSeparator = isSeparator || (*pinyin) == '\'' || (strcmp(im->uniqueName, "shuangpin-libpinyin") == 0 && (*pinyin) == ' ');
+                *pinyin = 0;
+                pinyin++;
+            }
 
             if (*lastpos != '\0') {
                 char* result = NULL;
                 boolean isshuangpin = false;
                 if (strcmp(im->uniqueName, "sunpinyin") == 0) {
                     FCITX_DEF_MODULE_ARGS(args, lastpos, &isshuangpin);
-                    result = FcitxSunPinyinInvokeGetFullPinyin(
-                        cloudpinyin->owner, args);
+                    result = FcitxSunPinyinInvokeGetFullPinyin(cloudpinyin->owner, args);
                 } else if (strcmp(im->uniqueName, "shuangpin") == 0) {
                     isshuangpin = true;
                     result = FcitxPinyinSP2QP(cloudpinyin->owner, lastpos);
                 }
                 if (isshuangpin) {
                     if (result) {
-                        if (plength + strlen(result) < MAX_USER_INPUT) {
+                        if (plength + strlen(result) + (engine[cloudpinyin->config.source].supportSeparator ? 1 : 0) < MAX_USER_INPUT) {
                             strcat(p + plength, result);
                             plength += strlen(result);
+                            if (engine[cloudpinyin->config.source].supportSeparator) {
+                                strcat(p + plength, "'");
+                                plength += 1;
+                            }
                             free(result);
                         } else {
                             p[hzlength] = '\0';
@@ -898,18 +919,26 @@ char *GetCurrentString(FcitxCloudPinyin* cloudpinyin, char **ascii_part)
                         }
                     }
                 } else {
-                    if (plength + strlen(lastpos) < MAX_USER_INPUT) {
+#define PINYIN_USE_SEPARATOR_CASE (isSeparator && engine[cloudpinyin->config.source].supportSeparator)
+                    if (plength + strlen(lastpos) + (PINYIN_USE_SEPARATOR_CASE ? 1 : 0) < MAX_USER_INPUT) {
                         strcat(p + plength, lastpos);
                         plength += strlen(lastpos);
+                        if (PINYIN_USE_SEPARATOR_CASE) {
+                            strcat(p + plength, "'");
+                            plength += 1;
+                        }
                     } else {
                         p[hzlength] = '\0';
                         break;
                     }
                 }
+
+                isSeparator = false;
             }
-            lastpos = pinyin + 1;
+            lastpos = pinyin;
+        } else {
+            pinyin ++;
         }
-        pinyin ++;
     } while(endflag);
     free(string);
     /* no pinyin append, return NULL for off it */
@@ -917,6 +946,9 @@ char *GetCurrentString(FcitxCloudPinyin* cloudpinyin, char **ascii_part)
         *ascii_part = NULL;
         return NULL;
     } else {
+        if (plength >= 1 && p[plength - 1] == '\'') {
+            p[plength - 1] = '\0';
+        }
         char *res = strdup(p);
         *ascii_part = res + hzlength;
         return res;
